#baseDir<-"../../PhD/_paper_XyDye/imageAnalysis_supplement";source(paste0(baseDir,"/step2_computeDistances.r"))

# computes ndbi values for each cut surface image

# requires:
#  config.r (measurementFile)
#  images/02_masks/xx.png

# creates:
#  images/04_distances/xx.png

library("readbitmap")
library("png")

if(!exists("baseDir")) baseDir <- getwd()
source(paste0(baseDir,"/config.r"))
experimentInfo <- read.table(paste0(baseDir,'/',config.measurementFile), sep='\t', header=TRUE)


fileList <- dir(paste0(baseDir,"/images/02_masks/"), full.names=TRUE)

for(n in 1:nrow(experimentInfo)){
	
	# load the mask file for the nth cut surface image
	filepath <- paste0(baseDir,"/images/02_masks/", experimentInfo$SegmentID[n],".png")
	mask <- read.bitmap(filepath)
	# the mask files are expected to use three colors:
	# black (mask value = 0): outside of the xylem
	# gray ( 0 < mask value < 1) : inside the xylem, but not to be included in final analysis (woodchips, etc..)
	# white (mask value = 1): inside the xylem
	
	# next line can be commented ot in case you managed to create a one channel mask
	mask <- mask[,,1] # selecting one of the three rgb channels
	
	# for the distance computation we will only differentiate by 
	#   inside the xylem (mask value = 1) and outside of the xylem (mask value = 0)
	mask[mask>0] <- 1 # grey pixels will be treated as white (inside xylem) pixels
	
	#--- this whole part is just for visualisation of mask and original image
	displayMask <- read.bitmap(filepath)[,,1]
	displayMask[displayMask==1] <- NA
	picFileName <- gsub("02_masks","01_photos",filepath)
	picFileName <- gsub("\\.png",".jpg",picFileName)	
	pic <- read.bitmap(picFileName)
	plot(NA, xlim=c(0,1), ylim=c(0,1), xaxs='i',yaxs='i')
	rasterImage(pic, 0,0,1,1)
	par(new=TRUE)
	# the image() function works in strange ways, so we have to rearrange 
	# our mask matrix a liitle bit before we can plot the mask on top of
	# the previously plotted raster
	flippedMask <-t(displayMask)[,nrow(mask):1]
	image(flippedMask)
	#-----------------------------------------
	
	cat("compute distance file for",experimentInfo$SegmentID[n] ,'\n')
	
	# count how many pixels around each pixel are outside of the xylem
	outsideCount <- mask*NA
	for(x in 2:(nrow(mask)-1)){
		for(y in 2:(ncol(mask)-1)){
			outsideCount[x,y] <- sum(mask[x+(-1:1), y+(-1:1)]<1)			
		}
	}

	# get x and y coordinates of each pixel on the xylem border
	border <- which(outsideCount>0 & outsideCount<9)
	borderYs <- floor((border-1)/nrow(mask))+1
	borderXs <- (border-1)%%nrow(mask)+1

	# determine each inside-xylem-pixel's distance to the xylem border
	d <- mask*NA
	for(x in 2:(nrow(mask)-1)){
		for(y in 2:(ncol(mask)-1)){
			if(!mask[x,y]) next
			# compute the pixels distance to each pixel on the xylem border 
			dists <- sqrt((x-borderXs)^2 + (y-borderYs)^2)
			
			#select the minimum distance as the pixel's xylem border distance
			d[x,y] <- min(dists)

		}
	}
	dout <- d
	dout <- d/max(d,na.rm=TRUE)
	dout[is.na(d)] <- 0
	
	png(gsub("02_masks","04_distances",filepath), width=ncol(mask), height=nrow(mask))
		par(mar=c(0,0,0,0))
		plot.new()
		rasterImage(dout,-0.04,-0.04,1.04,1.04, xaxs='i', yaxs='i')	
	dev.off()	
}


