#baseDir<-"../../PhD/_paper_XyDye/imageAnalysis_supplement";source(paste0(baseDir,"/step4_computeVelocities.r"))

# computes velocity distributions from blue stained area fractions and creates Fig.05

# requires:
#  config.r
#  result_step3_stainedAreaFractions.csv

# creates:
#  fig05.pdf

if(!exists("baseDir")) baseDir <- getwd()
source(paste0(baseDir,"/config.r"))

experimentInfoFile <- paste0(baseDir,'/',config.measurementFile)

experimentInfoRaw <- readLines(experimentInfoFile)

experimentInfoMeta <- list()
for(n in grep("^#>",experimentInfoRaw)){
	rawEntry <- unlist(strsplit(gsub("(^#>)|\\\t.*","",experimentInfoRaw[n]),'='))
	
	if(rawEntry[1]=="breakThroughTime"){
		value <- sum(as.numeric(unlist(strsplit(rawEntry[2],':'))) * c(3600,60,1))
	}else{
		value <- as.numeric(rawEntry[2])
	}
	experimentInfoMeta[[rawEntry[1]]] <- value
}

experimentInfo <- read.table(experimentInfoFile, sep='\t', header=TRUE)
cumL <- cumsum(experimentInfo$SegmentLength) - experimentInfo$SegmentLength[1]

t       <- experimentInfoMeta$breakThroughTime  #[s]
Vout    <- experimentInfoMeta$Vout              #[cm**3]
LPhloem <- experimentInfoMeta$phloemThickness  #[mm], thickness of phloem

rXylem <- min(experimentInfo$SegmentDiameter)/2 - LPhloem #[mm]
rXylem <- rXylem/1000 #[m]
A <- (rXylem^2*pi) #[m**2]

Q <- Vout/t  # [cm**3/s]
Q <- Q*86400 # [cm**3/d]
Q <- Q/1e6   # [m**3/d]

q <- Q/A # [m/d]


bFracs <- read.table(paste0(baseDir,"/result_step3_stainedAreaFractions.csv"), header=TRUE, sep='\t')
dClasses <- ncol(bFracs)
dCols <- c(NA,rev(hcl.colors(dClasses,"Plasma")))

vFracs <- NULL
for(n in 1:dClasses){
	vFrac <- -c(diff(bFracs[,n]),0)
	
	# make sure there are no negative fractions
	vFrac[vFrac<0] <- 0
	
	# apply some minor smoothing
	f <- 2
	mask <- (f+1):(length(vFrac)-f)
	vFrac[mask] <- filter(vFrac, rep(1/f,f))[mask]
	

	# append distance class results to overall results
	vFracs <- cbind(vFracs, vFrac)
}
vFracs <- vFracs/sum(vFracs)
vFrac <- rowSums(vFracs)
vFrac[mask] <- filter(vFrac, rep(1/f,f))[mask]
vFrac <- vFrac/sum(vFrac)

#--------------------fig05-----------------------------------------------
pdf(paste0(baseDir,"/fig05.pdf"), height=8.6/2.54, width=8.6/2.54)	
	
	layout(rbind(c(1,1,2),c(3,4,4)), widths=c(3,1.7,0.8),heights=c(3.5,3))
	
	#--sub plot (a)-----------
	
	par(mar=c(2.2,2.5,2.5,0.3), mgp=c(1.25,0.3,0), las=1, tcl=-0.25, cex=0.7)
	
	#---------------
	vVals <- cumL/1000 * 86400/2560
	plot(vVals, vFrac*100, pch=NA, ylab="Frequency [%]", xlab="",
		 xlim=c(0, max(vVals)*1.04), ylim=c(0,max(vFrac*115)), yaxs='i', xaxs='i', xaxt='n')
	abline(v=vVals, lty=2, col="#cccccc")
	
	mtext(1, line=1, text="Normalized flux and tracer velocity [-]",cex=par("cex"))
	
	text(par("usr")[2],par("usr")[4],labels="(a)",adj=c(1.1,1.2),font=2)
	
	secondaryX <- seq(0,6,1)
	
	axis(3)
	
	axis(1, at=secondaryX*q, labels=secondaryX)	
	mtext(3,line=par("mgp")[1], text="Volumetric flux (q) and tracer velocity (u) [m/d]", cex=par("cex"))
   
	vMeans <- NULL
	vStats <- NULL
	vMaxs <- NULL
	vMins <- NULL
	for(n in 1:ncol(vFracs)){
		vMean <- sum(vVals*vFracs[,n])/sum(vFracs[,n])		
		vMeans <- c(vMeans, vMean)
		
		mask <- vFracs[,n]>0
		
		cumFrac <- cumsum(vFracs[,n])
		cumFrac <- cumFrac/max(cumFrac)
		stats <- approx(cumFrac, vVals, xout=c(0.25,0.5,0.75))$y
		
		vStats <- rbind(vStats, stats)
		vMaxs <- c(vMaxs,max(vVals[mask]))
		vMins <- c(vMins,min(vVals[mask]))
	}
	
	vMeanTracer <- sum(vMeans*colSums(vFracs))
	for(n in 1:ncol(vFracs)){
		y0 <- rowSums(cbind(0,vFracs)[,1:n,drop=FALSE])*100
		y1 <- rowSums(vFracs[,1:n,drop=FALSE])*100
		polygon(c(vVals,rev(vVals)), c(y0, rev(y1)), col= dCols[n+1], border=NA)
	}
	
	for(n in 1:ncol(vFracs)){
		abline(v=vMeans[n], lwd=3)
		abline(v=vMeans[n], col=dCols[n+1], lwd=1.5)	
		y <- if(n==3) 14.4 else 16.1
		points(vMeans[n],y, col=dCols[n+1], pch=15,xpd=TRUE, cex=2)		
		text(vMeans[n], y, labels=parse(text = paste0("bar(u[",n,"])")), adj=c(0.5,0.5), cex=0.8, xpd=TRUE, col=if(n<4) "black" else "white")
	}
	
	abline(v=vMeanTracer, lwd=4)
	abline(v=vMeanTracer, lwd=2, col="#aaaaaa")
	points(vMeanTracer, 4, cex=3, pch=15, col="#aaaaaa")
	text(vMeanTracer, 4, labels=expression(bar(u)), adj=c(0.5, 0.5), cex=1.25)

	abline(v=q, lwd=4)
	abline(v=q, lwd=2, col="white", lty=3)
	points(q, 4, cex=3, pch=15)
	text(q, 4.5, labels="q", adj=c(0.5, 0.5), cex=1.25, col="white")
	
	#---Xylem depth class legend for (a)------------
	mar=par("mar")
	par(mar=c(2,0,4,0))
	plot.new()
	points(rep(1,dClasses), rep(0.5,dClasses), cex=(dClasses:1)*4.5, col=dCols[-1], pch=20)	
	text(c(0.94,0.75,0.57,0.40,0.23,0.06), 0.5, dClasses:1, col=rep(c('white','black'), each=3))
	text(0.5, 1.15, labels="Xylem\ndepth\nclasses", cex=0.75, xpd=TRUE)
	par(mar=mar)
	
	
	#--sub plot (b)-----------

	par(mar=par("mar")*c(1,1,0,1)+c(0,0,0.5,0), mgp=c(c(1.2,0.3,0)))
	matplot(vVals, vFracs*100,type='l', lty=1, col=dCols[-1], lwd=2, ylab='', xlab=expression("Tracer velocity [m/d]"))
	mtext(2, line=par("mgp")[1], text="Frequency [%]", cex=par("cex"), las=0)
	text(28.5,par("usr")[4],labels="(b)",adj=c(0.5,1.2), font=2)
	
	text(10,6,"outer xylem")	
	text(7,3,"inner xylem")
	
	arrows(c(10,7),c(5.5,2.5),c(15,2),c(5,1.5), length=0.075)
	
	#--sub plot (c)-----------
	
	par(mar=par("mar")*c(1,1,4,1))
	r <- (dClasses:1)-0.5
	rMax <- dClasses
	plot(r/rMax*100, vMeans, type='l', lwd=2, ylim=c(0,27), xlim=c(100,0), ylab=expression("Tracer velocity [m/d]"), xlab="Xylem Radius [%]",xaxt='n')
	axis(1, at=seq(0,100,20), labels=FALSE)
	text(seq(0,100,20), -5, labels=seq(0,100,20), xpd=TRUE)
	points(r/rMax*100, vMeans, pch=0, col=dCols[-1], lwd=4, cex=0.6)
	text(5,par("usr")[4],labels="(c)",adj=c(0.5,1.2), font=2)
	
	rect(r/rMax*100-7, vStats[,1], r/rMax*100+7, vStats[,3], border=dCols[-1], lwd=2)
	arrows(r/rMax*100, vMeans, r/rMax*100, vMins, col=dCols[-1], lwd=2, angle=90, length=0.05)
	arrows(r/rMax*100, vMeans, r/rMax*100, vMaxs, col=dCols[-1], lwd=2, angle=90, length=0.05)
	
	par(mgp=c(2,0.2,0))
	axis(3, at= r/rMax*100, labels=1:dClasses)
	mtext(3, line=1, text="Xylem depth class", cex=0.7)

dev.off()

